"""
Colorix
	A module for working with colors

	FILE: 			colormodes.py
	DESCRIPTION: 	Color modes
	
	Angel Carias. 2020.
"""
import tkinter


class ColorMode:
	"""
	Base class for color modes
	"""
	def __init__(self):
		self.supportModes = ["rgb", 'cmyk', 'hex']
		
		self.RGB_SCALE = 255
		self.CMYK_SCALE = 100


class RGB(ColorMode):
	"""
	RGB management class
	"""
	def __init__(self, r: int, g: int, b: int):
		super(RGB, self).__init__()
		self.r = int(r)
		self.b = int(b)
		self.g = int(g)
		
		self.raw = (self.r, self.g, self.b)
	
	def toHex(self):
		"""Convert RGB to Hex"""
		return Hex(f'#{self.r:02x}{self.g:02x}{self.b:02x}')	

	def toCMYK(self, precise=False):
		"""Convert RGB to CMYK"""
		if (self.r, self.g, self.b) == (0, 0, 0): # to avoid zero div error
			return CMYK(0, 0, 0, self.CMYK_SCALE)
			
		r = self.r / self.RGB_SCALE
		g = self.g / self.RGB_SCALE
		b = self.b / self.RGB_SCALE
		
		k = 1 - max(r, g, b)
		
		c = (1 - r - k) / (1 - k)
		m = (1 - g - k) / (1 - k)
		y = (1 - b - k) / (1 - k)
		
		cs = self.CMYK_SCALE
		
		if precise:
			return CMYK(c * cs, m * cs, y * cs, k * cs)
		else:
			return CMYK(round(c * cs), round(m * cs), round(y * cs), round(k * cs))
		
	def render(self, width: int=200, height: int=200):
		"""Render current color"""
		gui = tkinter.Tk(className="Color Render")
		gui.geometry(f"{width}x{height}")
		
		gui.configure(bg=self.toHex().hexval)
		gui.mainloop()


class Hex(ColorMode):
	"""
	Hex management class
	"""
	def __init__(self, hexval: str):
		super(Hex, self).__init__()
		self.hexval = hexval
	
	def toRGB(self):
		"""Convert Hex to RGB"""
		# Remove '#'
		hx = self.hexval.lstrip('#')
		
		# Convert to RGB using integer to base 16
		r = int(hx[0:2], 16)
		g = int(hx[2:4], 16)
		b = int(hx[4:6], 16)
		return RGB(r, g, b)
		
	def toCMYK(self):
		"""Convert Hex to CMYK"""
		# Remove '#'
		hx = self.hexval.lstrip('#')
		
		# Convert to RGB using integer to base 16
		r = int(hx[0:2], 16)
		g = int(hx[2:4], 16)
		b = int(hx[4:6], 16)
		
		# Convert RGB to CMYK
		return RGB(r, g, b).toCMYK()

	def render(self, width: int=200, height: int=200):
		"""Render current color"""
		gui = tkinter.Tk(className="Color Render")
		gui.geometry(f"{width}x{height}")
		
		gui.configure(bg=self.hexval)
		gui.mainloop()


class CMYK(ColorMode):
	"""
	CMYK management class
	"""
	def __init__(self, c: int, m: int, y: int, k: int):
		super(CMYK, self).__init__()
		
		self.c = int(c)
		self.m = int(m)
		self.y = int(y)
		self.k = int(k)
		
		self.raw = (self.c, self.m, self.y, self.k)
	
	def toHex(self):
		"""Convert CMYK to Hex"""
		# Convert CMYK to RGB
		rgb = CMYK(self.c, self.m, self.y, self.k).toRGB()
		
		# Convert RGB to Hex
		return Hex(f'#{rgb.r:02x}{rgb.g:02x}{rgb.b:02x}')
		
	def toRGB(self, precise=False):
		"""Convert CMYK to RGB"""
		
		c = self.c / self.CMYK_SCALE
		m = self.m / self.CMYK_SCALE
		y = self.y / self.CMYK_SCALE
		k = self.k / self.CMYK_SCALE
		
		r = self.RGB_SCALE * (1 - c) * (1 - k)
		g = self.RGB_SCALE * (1 - m) * (1 - k)
		b = self.RGB_SCALE * (1 - y) * (1 - k)
		
		if precise:
			return RGB(r, g, b)
		else:
			return RGB(round(r), round(g), round(b))

	def render(self, width: int=200, height: int=200):
		"""Render current color"""
		gui = tkinter.Tk(className="Color Render")
		gui.geometry(f"{width}x{height}")
		
		gui.configure(bg=self.toHex().hexval)
		gui.mainloop()
		


"""
Colorix
	A module for working with colors

	FILE: 			utils.py
	DESCRIPTION: 	Utilities
	
	Angel Carias. 2020.
"""
import random
from colorix import colormodes
from colorix.colormodes import ColorMode


class ModeNotSupported(Exception):
    def __init__(self, color):
        message = f"Color mode '{color}' not supported."
        super(ModeNotSupported, self).__init__(message)


class Utils:
    def colorRandom(self, mode: str="rgb"):
        mode = mode.lower()
        cm = ColorMode()
        
        if mode == "rgb":
            rgb = [random.randint(0, cm.RGB_SCALE) for _ in range(3)]
            return colormodes.RGB(*rgb)
        elif mode == "cmyk":
            cmyk = [random.randint(0, cm.CMYK_SCALE) for _ in range(4)]
            return colormodes.CMYK(*cmyk)
        elif mode == "hex":
            rgb = [random.randint(0, cm.RGB_SCALE) for _ in range(3)]
            return colormodes.RGB(*rgb).toHex()
        else:
            raise ModeNotSupported(mode)



from setuptools import setup
import re

with open('colorix\__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

print("Setup for version: ", version)

requirements = []

with open("README.md", "r") as f:
    readme = f.read()

setup(name="colorix",
      packages=["colorix"],
      author='Tekgar',
      author_email=None,
      version=version,
      description="A Python3 module for colors",
      long_description=readme,
      long_description_content_type="text/markdown",
      install_requires=requirements,
      python_requires=">=3.6",
      url="https://github.com/angelCarias/colorix",
      download_url="https://github.com/angelCarias/colorix/archive/v0.1.5.tar.gz"
      )
